package cn.skcks.docking.wx.services.mp.service;

import cn.hutool.core.date.DateUtil;
import cn.skcks.docking.wx.common.page.PageWrapper;
import cn.skcks.docking.wx.orm.entity.MpApp;
import cn.skcks.docking.wx.orm.mapper.MpAppMapper;
import cn.skcks.docking.wx.services.mp.vo.verify.SignatureVo;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.micrometer.common.util.StringUtils;
import jakarta.annotation.PostConstruct;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.api.impl.WxMpServiceImpl;
import me.chanjar.weixin.mp.bean.message.WxMpXmlMessage;
import me.chanjar.weixin.mp.bean.message.WxMpXmlOutMessage;
import me.chanjar.weixin.mp.config.impl.WxMpDefaultConfigImpl;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ForkJoinPool;

@Slf4j
@Service
@RequiredArgsConstructor
public class MpAppService {
    private final MpAppMapper mpAppMapper;

    private final Map<String, WxMpService> serviceMap = new ConcurrentHashMap<>();

    @PostConstruct
    private void init(){
        LambdaQueryChainWrapper<MpApp> queryWrapper = new LambdaQueryChainWrapper<>(mpAppMapper);
        queryWrapper.ne(MpApp::getEnable, false);
        List<MpApp> mpApps = queryWrapper.list();

        new ForkJoinPool(Runtime.getRuntime().availableProcessors()).execute(() -> {
            mpApps.parallelStream().forEach(mpApp -> {
                WxMpDefaultConfigImpl config = new WxMpDefaultConfigImpl();
                config.setAppId(mpApp.getAppId());
                config.setSecret(mpApp.getAppSecret());
                config.setToken(mpApp.getToken());
                config.setAesKey(mpApp.getEncodingAesKey());

                WxMpService service = new WxMpServiceImpl();
                service.setWxMpConfigStorage(config);

                serviceMap.put(mpApp.getAppId(), service);
            });
        });
    }

    public void service(HttpServletRequest request, HttpServletResponse response, String appId, SignatureVo signatureVo) throws IOException {
        response.setContentType("text/html;charset=utf-8");
        response.setStatus(HttpServletResponse.SC_OK);

        WxMpService wxMpService = serviceMap.get(appId);
        if(ObjectUtils.anyNull(appId, signatureVo, wxMpService)){
            response.getWriter().println("非法请求");
            return;
        }

        if(!wxMpService.checkSignature(signatureVo.getTimestamp(), signatureVo.getNonce(), signatureVo.getSignature())){
            response.getWriter().println("非法请求");
            return;
        }


        // 签名验证请求
        String echostr = request.getParameter("echostr");
        if(StringUtils.isNotBlank(echostr)){
            response.getWriter().println(echostr);
            return;
        }

        String encryptType = StringUtils.isBlank(request.getParameter("encrypt_type")) ?
                "raw" :
                request.getParameter("encrypt_type");

        if ("raw".equals(encryptType)) {
            // 明文传输的消息
            WxMpXmlMessage inMessage = WxMpXmlMessage.fromXml(request.getInputStream());
            // WxMpXmlOutMessage outMessage = wxMpMessageRouter.route(inMessage);
            // if(outMessage == null) {
            //     //为null，说明路由配置有问题，需要注意
            //     response.getWriter().write("");
            // }
            // response.getWriter().write(outMessage.toXml());

            log.debug("\n[↓] 接收消息: \n{}", inMessage);

            WxMpXmlOutMessage outMessage = WxMpXmlOutMessage.TEXT()
                    .toUser(inMessage.getFromUser())
                    .fromUser(inMessage.getToUser())
                    .content(DateUtil.now())
                    .build();

            log.debug("\n[↑] 响应消息: \n{}", outMessage);
            response.getWriter().write(outMessage.toXml());
            return;
        }

        // if ("aes".equals(encryptType)) {
        //     // 是aes加密的消息
        //     String msgSignature = request.getParameter("msg_signature");
        //     WxMpXmlMessage inMessage = WxMpXmlMessage.fromEncryptedXml(request.getInputStream(), wxMpConfigStorage, timestamp, nonce, msgSignature);
        //     WxMpXmlOutMessage outMessage = wxMpMessageRouter.route(inMessage);
        //     if(outMessage == null) {
        //         //为null，说明路由配置有问题，需要注意
        //         response.getWriter().write("");
        //     }
        //     response.getWriter().write(outMessage.toEncryptedXml(wxMpConfigStorage));
        //     return;
        // }

        response.getWriter().println("不可识别的加密类型");
        // return;
    }

    /**
     * 获取所有公众号列表
     *
     * @author shikong
     * @since 2024-12-07
     */
    public List<MpApp> getMpAppList(){
        return mpAppMapper.selectList(null);
    }


    /**
     * 分页查询公众号列表
     *
     * @author shikong
     * @since 2024-12-07
     */
    public PageWrapper<MpApp> getMpAppPageList(Integer pageNum, Integer pageSize){
        // try(Page<Object> ignored = PageHelper.startPage(pageNum, pageSize)){
        //     return new PageInfo<>(mpAppMapper.selectList(null));
        // }

        Page<MpApp> mpAppPage = mpAppMapper.selectPage(new Page<>(pageNum, pageSize), null);
        return PageWrapper.of(mpAppPage);
    }
}
